@@ -33,6 +33,10 @@ def test_train_cls():
     run(f'yolo train classify model={CFG}-cls.yaml data=imagenet10 imgsz=32 epochs=1')
 
 
+def test_train_pose():
+    run(f'yolo train pose model={CFG}-pose.yaml data=coco8-pose.yaml imgsz=32 epochs=1')
+
+
 # Val checks -----------------------------------------------------------------------------------------------------------
 def test_val_detect():
     run(f'yolo val detect model={MODEL}.pt data=coco8.yaml imgsz=32')
@@ -46,6 +50,10 @@ def test_val_classify():
     run(f'yolo val classify model={MODEL}-cls.pt data=imagenet10 imgsz=32')
 
 
+def test_val_pose():
+    run(f'yolo val pose model={MODEL}-pose.pt data=coco8-pose.yaml imgsz=32')
+
+
 # Predict checks -------------------------------------------------------------------------------------------------------
 def test_predict_detect():
     run(f"yolo predict model={MODEL}.pt source={ROOT / 'assets'} imgsz=32 save save_crop save_txt")
@@ -63,6 +71,10 @@ def test_predict_classify():
     run(f"yolo predict model={MODEL}-cls.pt source={ROOT / 'assets'} imgsz=32 save save_txt")
 
 
+def test_predict_pose():
+    run(f"yolo predict model={MODEL}-pose.pt source={ROOT / 'assets'} imgsz=32 save save_txt")
+
+
 # Export checks --------------------------------------------------------------------------------------------------------
 def test_export_detect_torchscript():
     run(f'yolo export model={MODEL}.pt format=torchscript')
@@ -76,6 +88,10 @@ def test_export_classify_torchscript():
     run(f'yolo export model={MODEL}-cls.pt format=torchscript')
 
 
+def test_export_classify_pose():
+    run(f'yolo export model={MODEL}-pose.pt format=torchscript')
+
+
 def test_export_detect_edgetpu(enabled=False):
     if enabled and LINUX:
         run(f'yolo export model={MODEL}.pt format=edgetpu')
